classdef tag < handle
% Abstract class for plugins that do tagging
    
    %% CONFIGURATION PARAMETERS
    properties
        % lower threshold used to detect threshold crossings. Set to nan by
        % default because the user needs to make a reasonable choice.
        TAG_THRESHOLD = nan;
        % upper treshold used to filter out specific data
        TAG_THRESHOLD_UP = [];
        % what data type is the plugin expected to be given? 0 = integer, 1 = double.
        TAG_DATA_TYPE = 0;
        % how many samples have to lie between each index 
        % (threshold crossing)? This creates something like an 
        % "non-overlap" window. A value of 0 means that threshold crossings 
        % can be in consecutive samples.
        TAG_SAMPLES_BETWEEN_INDICES = 0;
        % should data around the indices we returned?
        TAG_EXTRACT_BUFFERS = 1;
        % how many data points before and after the threshold should be
        % extracted?
        TAG_PRESAMPLES = 512;
        TAG_POSTSAMPLES = 512;
        % should overlapping buffers be discarded?
        TAG_RETURN_WHOLE_BUFFERS_ONLY = 1;
        % should the data be returned in INT16?
        TAG_RETURN_DATA_AS_INT16 = 1;
        % What is the dynamic range of the original data? 5 volts is a good
        % approximation.
        TAG_TRANSFORM_DATA_DYNAMIC_RANGE = 5;
        % By default tag_data() will only work if you pass it a one
        % dimensional data array (row vector of size (1, n)). However, you
        % can override this limit by settting TAG_MULTIDIM_DATA_MODE_ENABLED
        % to 1
        TAG_MULTIDIM_DATA_MODE_ENABLED = 0;
        % If TAG_MULTIDIM_DATA_MODE_ENABLED is set to 1, you need to say
        % which one of the many possible data channels should be used to
        % run tag_data() on? tag_data() will operate on this channel
        % and then extract all corresponding data snippets from all
        % channels.
        TAG_MULTIDIM_USE_CHANNEL = 1;
        % TODO: document what this is doing so someone understands it.
        % only used in transform_indices(). 
        % clean up multiple onsets / offsets?
        % used in fb_abstract.tag.transform_indices()
        TAG_TRANSFORM_CLEAN = 1;
        % don't do any real tagging, just return one fake index per file
        % useful if you need a list of files to use for post processing
        TAG_RETURN_FAKE_INDICES = 0;
        % Define minimal window size of SIGNAL window:
        % i.e., if there is a threshold crossing @ 100 and one @ 250, but
        % the window size needs to be at least 300, then these two will be
        % merged (and the noise window will be removed). If the merged 
        % window is still too small, kill the signal window.
        % The default value is 0.
        TAG_MIN_WINDOW_SIZE = 0;
    end
    
    %% OUTPUT VARIABLES
    properties (SetAccess = protected, GetAccess = public)
        TAG_INDICES
        TAG_INDICES_OFF
        TAG_FOUND
        TAG_DATA
        TAG_FILE_LENGTH_SAMPLES
        % flag to indicated whether tagging failed and has been aborted
        TAG_ABORTED = 0;
        % Y axis data plotted to TAG_RAW_OUTPUT_HANDLE
        TAG_RAW_OUTPUT = [];        
    end
    
    properties (SetAccess = protected, GetAccess = public, Transient = true)
        % object holding the vertical lines in GUI TAGGING mode
        TAG_GUI_OBJECT = [];
        % context menu for GUI TAGGING
        TAG_GUI_contextmenu = [];
        % indicate whether tag_data is running
        TAG_DATA_RUNNING = 0;
    end
    
    %% INTERFACE PARAMETERS CHARCTERISTIC FOR TAGGING - DO NOT CHANGE!!
    properties (Constant = true)
        % mark this class as a tag plugin class
        TAG_PLUGIN = 1;
        % currently supported data types
        TAG_SUPPORTED_DATA_TYPES = [0 1];
        % currently support buffer extraction methods
        TAG_SUPPORTED_EXTRACT_BUFFERS = [0 1];
        % currently supported multidimension modes
        TAG_SUPPORTED_MULTIDIM_DATA_MODE_ENABLED = [0 1];
    end
    
    %% non-public properties
    properties (SetAccess = protected, GetAccess = protected)
        TAG_RAW_OUTPUT_LINE_WIDTH = 1;
        % property set to check whether we should quit tag_data()
        TAG_QUIT = 0;
        % is GUI TAGGING activated?
        TAG_GUI_ACTIVATED = 0;
    end
    
    properties (SetAccess = protected, GetAccess = protected, Transient = true)
        % handle used for RAW data output (fb_plugin_detect_song_harmonics)
        TAG_RAW_OUTPUT_HANDLE
        % did we setup the RAW_OUTPUT_HANDLE?
        TAG_RAW_OUTPUT_SETUP = 0;
        % listener for NewFile
        TAG_listener_NewFile = [];
        % listener for NewROI
        TAG_listener_NewROI = [];
        % listener to hide TAG_RAW_OUTPUT_HANDLE
        TAG_listener_hide_handles
        % listener to show TAG_RAW_OUTPUT_HANDLE
        TAG_listener_show_handles
    end
    
    %% public methods
    methods
        
        %% constructor
        function obj = tag()
            
            % make sure results field have default values (important for
            % multicore saving / loading of plugins)
            initialize_result_fields(obj);

            % registers the listeners for showing / hiding
            % TAG_RAW_OUTPUT_HANDLE
            tag_createListeners_TAG_RAW_OUTPUT_HANDLE(obj);
            
        end
        
        %% custom delete function -  make sure we delete the
        % TAG_RAW_OUTPUT_HANDLE handle in case it is used.
        function delete(obj)
            if ishandle(obj.TAG_RAW_OUTPUT_HANDLE)
                delete(obj.TAG_RAW_OUTPUT_HANDLE);
            end
        end
        
        %% show / hide the analog values
        function TagListenShowHide(obj, src, evnt) %#ok<INUSD>
            fb_abstract.GUI_handle.invert_visibility(obj.TAG_RAW_OUTPUT_HANDLE);
        end
        
        %% SET METHODS
        function set.TAG_THRESHOLD(obj, TAG_THRESHOLD)
            if TAG_THRESHOLD < 0
                disp('Be careful: you''ve set your threshold to be smaller then zero.');
            end
            obj.TAG_THRESHOLD = TAG_THRESHOLD;
        end
        
        function set.TAG_DATA_TYPE(obj, TAG_DATA_TYPE)
            check_setting(obj, TAG_DATA_TYPE, 'TAG_SUPPORTED_DATA_TYPES');
            obj.TAG_DATA_TYPE = TAG_DATA_TYPE;
        end
        
        function set.TAG_EXTRACT_BUFFERS(obj, TAG_EXTRACT_BUFFERS)
            check_setting(obj, TAG_EXTRACT_BUFFERS, 'TAG_SUPPORTED_EXTRACT_BUFFERS');
            obj.TAG_EXTRACT_BUFFERS = TAG_EXTRACT_BUFFERS;
        end
        
        function set.TAG_MULTIDIM_DATA_MODE_ENABLED(obj, TAG_MULTIDIM_DATA_MODE_ENABLED)
            check_setting(obj, TAG_MULTIDIM_DATA_MODE_ENABLED, 'TAG_SUPPORTED_MULTIDIM_DATA_MODE_ENABLED');
            obj.TAG_MULTIDIM_DATA_MODE_ENABLED = TAG_MULTIDIM_DATA_MODE_ENABLED;
        end
        
        function set.TAG_MULTIDIM_USE_CHANNEL(obj, TAG_MULTIDIM_USE_CHANNEL)
            fb_abstract.basic.chk_pos_scalar(TAG_MULTIDIM_USE_CHANNEL, 'TAG_MULTIDIM_USE_CHANNEL');
            obj.TAG_MULTIDIM_USE_CHANNEL = TAG_MULTIDIM_USE_CHANNEL;
        end
        
        function set.TAG_QUIT(obj, TAG_QUIT)
            obj.TAG_ABORTED = TAG_QUIT;
            obj.TAG_QUIT = TAG_QUIT;
        end
        
        function set.TAG_GUI_ACTIVATED(obj, tag_gui_activated)

            % don't do anything if the value doesn't change
            if obj.TAG_GUI_ACTIVATED == tag_gui_activated
                return;
            end
            
            if obj.TAG_GUI_ACTIVATED == 0
                tag_GUI_activate(obj);
            else
                tag_GUI_deactivate(obj);
            end
            
            obj.TAG_GUI_ACTIVATED = tag_gui_activated;
            
        end
        
    end
    
    %% methods which CAN NOT be redefined.
    methods (Sealed = true)
        tag_data(obj, data)
        tag_clear(obj)
        tag_plot(obj)
        tag_GUI_activate(obj)
        tag_GUI_deactivate(obj)
        tag_GUI_setup_menu(obj, contextmenu)
    end
    
    methods (Sealed = true, Hidden = true)
        tag_class_plot(obj)
    end
    
    %% hidden methods
    methods (Access = protected, Hidden = true)
        plugin_tag_function(obj, data)
        plugin_tag_plot(obj)
        [indices, indices_off] = plugin_tag_find_indices(obj, transformed_data)
    end
    
    %% SEALED methods that can be used inside custom written plugin_tag_function() functions.
    methods (Access = protected, Hidden = true, Sealed = true)
        transformed_data = find_indices_pre(obj, data)
        remove_overlapping_indices(obj)
        extract_data(obj, data, nbr_TAG_INDICES)
        transform_indices(obj, indices_on_orig, indices_off_orig)
        [indices, indices_off] = clean_up_indices(obj, indices, indices_off)
    end
    
    
    %% private hidden methods
    methods (Access = private, Hidden = true)
        run_checks(obj, data)
        [indices, indices_off] = find_indices(obj, data)
        check_setting(obj, value, valids)
        initialize_result_fields(obj)
        tag_createListeners_TAG_RAW_OUTPUT_HANDLE(obj);
    end
    
    %% hidden methods overriding plugin specific methods
    methods (Access = protected, Hidden = true, Sealed = true)
        tag_setup_TAG_RAW_OUTPUT_HANDLE(obj)
    end

    
    %% NOT USED loadobj and saveobj methods - we use transient properties
    % for listeners and handles - that takes care of all potential problems
    
    % the private loadobj & saveobj methods will not be used for saving or
    % loading an object. only old objects that were previously saved with
    % loadobj and saveobj, will use these methods. This is a dirty hack
    % which prevents that we get ugly error messages while loading old
    % objects.
    
    methods (Static = true, Access = private)
        obj = loadobj_notused(S)
        obj = loadobj(S)
    end
    
    methods (Access = private)
        S = saveobj_notused(obj)
        S = saveobj(obj)
    end
    
end
%% EOF
